"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = void 0;
const package_1 = require("./utils/package");
const fs_1 = require("fs");
const typescript_1 = require("./utils/typescript");
const controller_1 = require("./utils/controller");
const util_1 = require("./utils/util");
const openapi_1 = require("./utils/openapi");
const omit_by_1 = require("./utils/omit-by");
const lodash_1 = require("lodash");
function setMethodSchema(schema, path, httpMethodType, responseType, tag, methodParameterSchemas) {
    schema.paths[path] = schema.paths[path] || {};
    schema.paths[path][httpMethodType] = Object.assign({ tags: [tag.name], responses: { 200: (0, openapi_1.getResponseObject)(responseType) } }, (0, lodash_1.cloneDeep)(methodParameterSchemas));
}
function generate(options) {
    const schema = {
        openapi: "3.0.0",
        info: (0, package_1.getPackageInfo)(),
        paths: {},
        components: {},
        tags: [],
    };
    (0, typescript_1.getControllers)(options === null || options === void 0 ? void 0 : options.tsConfigFilePath).forEach((controller) => {
        var _a;
        const tag = (0, controller_1.getControllerTag)(controller);
        (_a = schema.tags) === null || _a === void 0 ? void 0 : _a.push(tag);
        const basePath = (0, controller_1.getControllerPath)(controller, options === null || options === void 0 ? void 0 : options.prefix);
        controller.getMethods().forEach((method) => {
            const methodDetails = (0, typescript_1.getMethodDetails)(method);
            if (!methodDetails)
                return;
            const methodParameterSchemas = (0, openapi_1.getMethodParameters)(methodDetails.parameters);
            const path = (0, util_1.getOpenapiPath)(basePath, methodDetails.path);
            const optionalParams = (0, lodash_1.orderBy)(methodParameterSchemas.parameters.filter((p) => p.in === "path" && !p.required), (p) => (0, lodash_1.findIndex)(path, p.name));
            if (!optionalParams.length)
                setMethodSchema(schema, path, methodDetails.httpMethodType, methodDetails.responseType, tag, methodParameterSchemas);
            else {
                methodParameterSchemas.parameters = methodParameterSchemas.parameters.filter((p) => p.in !== "path" || p.required);
                optionalParams.forEach((optionalParam) => {
                    const partialPath = path.split("/{" + optionalParam.name)[0];
                    if (!schema.paths[partialPath])
                        setMethodSchema(schema, partialPath, methodDetails.httpMethodType, methodDetails.responseType, tag, methodParameterSchemas);
                    optionalParam.required = true;
                    methodParameterSchemas.parameters.push(optionalParam);
                    setMethodSchema(schema, partialPath + "/{" + optionalParam.name + "}", methodDetails.httpMethodType, methodDetails.responseType, tag, methodParameterSchemas);
                });
            }
        });
    });
    schema.components["schemas"] = openapi_1.definitions;
    const swaggerSchema = (0, omit_by_1.omitDeepBy)((0, lodash_1.cloneDeep)(schema), (x, y) => {
        return y === "optional";
    });
    (0, fs_1.writeFileSync)((options === null || options === void 0 ? void 0 : options.filePath) || "./openapi.schema.json", JSON.stringify(swaggerSchema));
}
exports.generate = generate;
