"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMethodDetails = exports.getControllers = exports.isPrimitive = void 0;
const ts_morph_1 = require("ts-morph");
const isPrimitive = (type) => type.isBoolean() || type.isNumber() || type.isString();
exports.isPrimitive = isPrimitive;
function getControllers(tsConfigFilePath = process.cwd() + "/tsconfig.json") {
    const project = new ts_morph_1.Project({ tsConfigFilePath });
    const sourceFiles = project.getSourceFiles();
    const controllers = [];
    sourceFiles.forEach((s) => {
        s.getClasses().forEach((c) => {
            var _a;
            if ((_a = c.getName()) === null || _a === void 0 ? void 0 : _a.endsWith("Controller"))
                controllers.push(c);
        });
    });
    return controllers;
}
exports.getControllers = getControllers;
function getMethodDetails(method) {
    var _a, _b;
    const validRequestTypes = ["Post", "Get", "Delete", "Put", "Path", "Options", "Head"];
    const methodTypeDecorator = method.getDecorators().find((decorator) => validRequestTypes.includes(decorator.getName()));
    if (!methodTypeDecorator)
        return;
    const path = ((_a = methodTypeDecorator.getArguments()[0]) === null || _a === void 0 ? void 0 : _a.getLiteralValue()) || "";
    const httpMethodType = methodTypeDecorator.getName().toLowerCase();
    const description = (_b = method === null || method === void 0 ? void 0 : method.getJsDocs()[0]) === null || _b === void 0 ? void 0 : _b.getDescription();
    const responseType = method === null || method === void 0 ? void 0 : method.getReturnType();
    // if (responseType?.getTypeArguments()[0]) responseType = responseType?.getTypeArguments()[0];
    const parameters = method.getParameters();
    return { path, httpMethodType, description, responseType, parameters };
}
exports.getMethodDetails = getMethodDetails;
